#version 330
#extension GL_ARB_separate_shader_objects : require
void CalculateDiffuseLighting( const in vec3 i_normal_world,
    const in vec3 i_light_direction, const in vec3 i_light_color,
    out vec3 o_lighting_diffuse )
{

    float lambertTerm;
    {
        float dotProduct;
        {

            vec3 lightVector = -i_light_direction;
            dotProduct = dot( i_normal_world, lightVector );
        }




        lambertTerm = max( 0.0, dotProduct );
    }

    o_lighting_diffuse = lambertTerm * i_light_color;
}
void CalculateSpecularLighting( const in vec3 i_position_world, const in vec3 i_normal_world, const in vec3 i_camera_position,
    const in vec3 i_light_direction, const in vec3 i_light_color,
    const in float i_shininess,
    out vec3 o_lighting_specular )
{
    float specularAmount;
    {

        float dotProduct;
        {

            vec3 reflectedDirection = reflect( i_light_direction, i_normal_world );

            vec3 viewDirection = normalize( i_camera_position - i_position_world );

            dotProduct = dot( reflectedDirection, viewDirection );

            dotProduct = max( 0.0, dotProduct );
        }


        specularAmount = pow( dotProduct, i_shininess );
    }

    o_lighting_specular = i_light_color * specularAmount;
}



uniform mat4 g_Local_To_World;
uniform mat4 g_World_To_View;
uniform mat4 g_View_To_Screen;
layout( location = 0 ) in vec3 i_position;
layout( location = 1 ) in vec4 i_color;
layout( location = 2 ) in vec2 i_texcoord;
layout( location = 3 ) in vec3 i_normal;




layout( location = 0 ) out vec4 o_color;
layout( location = 1 ) out vec2 o_texcoord;
layout( location = 2 ) out vec3 o_normal;
layout( location = 3 ) out vec3 o_world_position;




void main()

{
	{
		vec4 position_world = vec4( i_position, 1.0 ) * g_Local_To_World;
		vec4 position_view = position_world * g_World_To_View;
		gl_Position = position_view * g_View_To_Screen;

		o_world_position = position_world.xyz;
	}

	{
		mat3 rotation_localToWorld = mat3(
		g_Local_To_World[0].xyz,
		g_Local_To_World[1].xyz,
		g_Local_To_World[2].xyz );
		o_normal = i_normal * rotation_localToWorld;
	}

	{
		o_color = i_color;
		o_texcoord = i_texcoord;
	}
}
